From 0e1083f182ca6980645b65ab918f3571031ca522 Mon Sep 17 00:00:00 2001
From: ogg.k.ogg.k <ogg.k.ogg.k@googlemail.com>
Date: Fri, 26 Dec 2008 11:26:23 +0000
Subject: [PATCH] Handle Ogg packets larger than (or equal to) 64 KB.

---
 modules/codec/kate.c   |   18 ++++++++++++------
 modules/codec/speex.c  |   22 +++++++++++++++-------
 modules/codec/theora.c |   30 ++++++++++++++++++++----------
 modules/codec/vorbis.c |   28 +++++++++++++++++++---------
 modules/demux/ogg.c    |    6 ++++--
 modules/mux/ogg.c      |   18 ++++++++++++------
 6 files changed, 82 insertions(+), 40 deletions(-)

diff --git a/modules/codec/kate.c b/modules/codec/kate.c
index 0e8a899..132c07b 100644
--- a/modules/codec/kate.c
+++ b/modules/codec/kate.c
@@ -505,13 +505,15 @@ static subpicture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
         uint8_t *p_extra;
 
         p_dec->fmt_in.p_extra =
-            realloc( p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra + kp.nbytes + 2 );
+            realloc( p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra + kp.nbytes + 4 );
         p_extra = (void*)(((unsigned char*)p_dec->fmt_in.p_extra) + p_dec->fmt_in.i_extra);
+        *(p_extra++) = kp.nbytes >> 24;
+        *(p_extra++) = kp.nbytes >> 16;
         *(p_extra++) = kp.nbytes >> 8;
         *(p_extra++) = kp.nbytes & 0xFF;
 
         memcpy( p_extra, kp.data, kp.nbytes );
-        p_dec->fmt_in.i_extra += kp.nbytes + 2;
+        p_dec->fmt_in.i_extra += kp.nbytes + 4;
 
         block_Release( *pp_block );
         p_sys->i_num_headers = ((unsigned char*)p_dec->fmt_in.p_extra)[0];
@@ -556,11 +558,13 @@ static int ProcessHeaders( decoder_t *p_dec )
     --i_extra;
 
     /* Take care of the initial Kate header */
-    kp.nbytes = *(p_extra++) << 8;
+    kp.nbytes = *(p_extra++) << 24;
+    kp.nbytes |= *(p_extra++) << 16;
+    kp.nbytes |= *(p_extra++) << 8;
     kp.nbytes |= (*(p_extra++) & 0xFF);
     kp.data = p_extra;
     p_extra += kp.nbytes;
-    i_extra -= (kp.nbytes + 2);
+    i_extra -= (kp.nbytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -582,11 +586,13 @@ static int ProcessHeaders( decoder_t *p_dec )
     /* parse all remaining header packets */
     for( i_headeridx = 1; i_headeridx < p_sys->ki.num_headers; ++i_headeridx )
     {
-        kp.nbytes = *(p_extra++) << 8;
+        kp.nbytes = *(p_extra++) << 24;
+        kp.nbytes |= *(p_extra++) << 16;
+        kp.nbytes |= *(p_extra++) << 8;
         kp.nbytes |= (*(p_extra++) & 0xFF);
         kp.data = p_extra;
         p_extra += kp.nbytes;
-        i_extra -= (kp.nbytes + 2);
+        i_extra -= (kp.nbytes + 4);
         if( i_extra < 0 )
         {
             msg_Err( p_dec, "header %d data corrupted", i_headeridx );
diff --git a/modules/codec/speex.c b/modules/codec/speex.c
index 547448c..d548afd 100644
--- a/modules/codec/speex.c
+++ b/modules/codec/speex.c
@@ -311,13 +311,15 @@ static void *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
 
         p_dec->fmt_in.p_extra =
             realloc( p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra +
-                     oggpacket.bytes + 2 );
+                     oggpacket.bytes + 4 );
         p_extra = ((uint8_t *)p_dec->fmt_in.p_extra) + p_dec->fmt_in.i_extra;
+        *(p_extra++) = oggpacket.bytes >> 24;
+        *(p_extra++) = oggpacket.bytes >> 16;
         *(p_extra++) = oggpacket.bytes >> 8;
         *(p_extra++) = oggpacket.bytes & 0xFF;
 
         memcpy( p_extra, oggpacket.packet, oggpacket.bytes );
-        p_dec->fmt_in.i_extra += oggpacket.bytes + 2;
+        p_dec->fmt_in.i_extra += oggpacket.bytes + 4;
 
         block_Release( *pp_block );
         p_sys->i_headers++;
@@ -359,11 +361,13 @@ static int ProcessHeaders( decoder_t *p_dec )
     i_extra = p_dec->fmt_in.i_extra;
 
     /* Take care of the initial Vorbis header */
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
     p_extra += oggpacket.bytes;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -379,11 +383,13 @@ static int ProcessHeaders( decoder_t *p_dec )
 
     /* The next packet in order is the comments header */
     oggpacket.b_o_s = 0;
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
     p_extra += oggpacket.bytes;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -1009,10 +1015,12 @@ static int OpenEncoder( vlc_object_t *p_this )
     pp_header[1] = "ENCODER=VLC media player";
     pi_header[1] = sizeof("ENCODER=VLC media player");
 
-    p_enc->fmt_out.i_extra = 3 * 2 + pi_header[0] + pi_header[1];
+    p_enc->fmt_out.i_extra = 4 * 2 + pi_header[0] + pi_header[1];
     p_extra = p_enc->fmt_out.p_extra = malloc( p_enc->fmt_out.i_extra );
     for( i = 0; i < 2; i++ )
     {
+        *(p_extra++) = pi_header[i] >> 24;
+        *(p_extra++) = pi_header[i] >> 16;
         *(p_extra++) = pi_header[i] >> 8;
         *(p_extra++) = pi_header[i] & 0xFF;
         memcpy( p_extra, pp_header[i], pi_header[i] );
diff --git a/modules/codec/theora.c b/modules/codec/theora.c
index 5817bf0..f5581ea 100644
--- a/modules/codec/theora.c
+++ b/modules/codec/theora.c
@@ -217,13 +217,15 @@ static void *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
 
         p_dec->fmt_in.p_extra =
             realloc( p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra +
-                     oggpacket.bytes + 2 );
+                     oggpacket.bytes + 4 );
         p_extra = ((uint8_t *)p_dec->fmt_in.p_extra) + p_dec->fmt_in.i_extra;
+        *(p_extra++) = oggpacket.bytes >> 24;
+        *(p_extra++) = oggpacket.bytes >> 16;
         *(p_extra++) = oggpacket.bytes >> 8;
         *(p_extra++) = oggpacket.bytes & 0xFF;
 
         memcpy( p_extra, oggpacket.packet, oggpacket.bytes );
-        p_dec->fmt_in.i_extra += oggpacket.bytes + 2;
+        p_dec->fmt_in.i_extra += oggpacket.bytes + 4;
 
         block_Release( *pp_block );
         p_sys->i_headers++;
@@ -265,11 +267,13 @@ static int ProcessHeaders( decoder_t *p_dec )
     i_extra = p_dec->fmt_in.i_extra;
 
     /* Take care of the initial Vorbis header */
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
     p_extra += oggpacket.bytes;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -354,11 +358,13 @@ static int ProcessHeaders( decoder_t *p_dec )
 
     /* The next packet in order is the comments header */
     oggpacket.b_o_s = 0;
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
     p_extra += oggpacket.bytes;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -377,10 +383,12 @@ static int ProcessHeaders( decoder_t *p_dec )
     /* The next packet in order is the codebooks header
      * We need to watch out that this packet is not missing as a
      * missing or corrupted header is fatal. */
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -702,7 +710,7 @@ static int OpenEncoder( vlc_object_t *p_this )
     theora_comment_init( &p_sys->tc );
 
     /* Create and store headers */
-    p_enc->fmt_out.i_extra = 3 * 2;
+    p_enc->fmt_out.i_extra = 3 * 4;
     for( i = 0; i < 3; i++ )
     {
         if( i == 0 ) theora_encode_header( &p_sys->td, &header );
@@ -713,9 +721,11 @@ static int OpenEncoder( vlc_object_t *p_this )
             realloc( p_enc->fmt_out.p_extra,
                      p_enc->fmt_out.i_extra + header.bytes );
         p_extra = p_enc->fmt_out.p_extra;
-        p_extra += p_enc->fmt_out.i_extra + (i-3)*2;
+        p_extra += p_enc->fmt_out.i_extra + (i-3)*4;
         p_enc->fmt_out.i_extra += header.bytes;
 
+        *(p_extra++) = header.bytes >> 24;
+        *(p_extra++) = header.bytes >> 16;
         *(p_extra++) = header.bytes >> 8;
         *(p_extra++) = header.bytes & 0xFF;
 
diff --git a/modules/codec/vorbis.c b/modules/codec/vorbis.c
index 4d079dc..f2cea1c 100644
--- a/modules/codec/vorbis.c
+++ b/modules/codec/vorbis.c
@@ -326,13 +326,15 @@ static void *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
 
         p_dec->fmt_in.p_extra =
             realloc( p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra +
-                     oggpacket.bytes + 2 );
+                     oggpacket.bytes + 4 );
         p_extra = (uint8_t *)p_dec->fmt_in.p_extra + p_dec->fmt_in.i_extra;
+        *(p_extra++) = oggpacket.bytes >> 24;
+        *(p_extra++) = oggpacket.bytes >> 16;
         *(p_extra++) = oggpacket.bytes >> 8;
         *(p_extra++) = oggpacket.bytes & 0xFF;
 
         memcpy( p_extra, oggpacket.packet, oggpacket.bytes );
-        p_dec->fmt_in.i_extra += oggpacket.bytes + 2;
+        p_dec->fmt_in.i_extra += oggpacket.bytes + 4;
 
         block_Release( *pp_block );
         p_sys->i_headers++;
@@ -374,11 +376,13 @@ static int ProcessHeaders( decoder_t *p_dec )
     i_extra = p_dec->fmt_in.i_extra;
 
     /* Take care of the initial Vorbis header */
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
     p_extra += oggpacket.bytes;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -414,11 +418,13 @@ static int ProcessHeaders( decoder_t *p_dec )
 
     /* The next packet in order is the comments header */
     oggpacket.b_o_s = 0;
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
     p_extra += oggpacket.bytes;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -435,10 +441,12 @@ static int ProcessHeaders( decoder_t *p_dec )
     /* The next packet in order is the codebooks header
      * We need to watch out that this packet is not missing as a
      * missing or corrupted header is fatal. */
-    oggpacket.bytes = *(p_extra++) << 8;
+    oggpacket.bytes = *(p_extra++) << 24;
+    oggpacket.bytes |= *(p_extra++) << 16;
+    oggpacket.bytes |= *(p_extra++) << 8;
     oggpacket.bytes |= (*(p_extra++) & 0xFF);
     oggpacket.packet = p_extra;
-    i_extra -= (oggpacket.bytes + 2);
+    i_extra -= (oggpacket.bytes + 4);
     if( i_extra < 0 )
     {
         msg_Err( p_dec, "header data corrupted");
@@ -885,11 +893,13 @@ static int OpenEncoder( vlc_object_t *p_this )
     /* Create and store headers */
     vorbis_analysis_headerout( &p_sys->vd, &p_sys->vc,
                                &header[0], &header[1], &header[2]);
-    p_enc->fmt_out.i_extra = 3 * 2 + header[0].bytes +
+    p_enc->fmt_out.i_extra = 3 * 4 + header[0].bytes +
        header[1].bytes + header[2].bytes;
     p_extra = p_enc->fmt_out.p_extra = malloc( p_enc->fmt_out.i_extra );
     for( i = 0; i < 3; i++ )
     {
+        *(p_extra++) = header[i].bytes >> 24;
+        *(p_extra++) = header[i].bytes >> 16;
         *(p_extra++) = header[i].bytes >> 8;
         *(p_extra++) = header[i].bytes & 0xFF;
         memcpy( p_extra, header[i].packet, header[i].bytes );
diff --git a/modules/demux/ogg.c b/modules/demux/ogg.c
index eadf1c0..71a3937 100644
--- a/modules/demux/ogg.c
+++ b/modules/demux/ogg.c
@@ -656,7 +656,7 @@ static void Ogg_DecodePacket( demux_t *p_demux,
         /* Backup the ogg packet (likely an header packet) */
         p_stream->p_headers =
             realloc( p_sav = p_stream->p_headers, p_stream->i_headers +
-                     p_oggpacket->bytes + (b_store_size ? 2 : 0) + (b_store_num_headers ? 1 : 0) );
+                     p_oggpacket->bytes + (b_store_size ? 4 : 0) + (b_store_num_headers ? 1 : 0) );
         if( p_stream->p_headers )
         {
             uint8_t *p_extra = p_stream->p_headers + p_stream->i_headers;
@@ -669,11 +669,13 @@ static void Ogg_DecodePacket( demux_t *p_demux,
             }
             if( b_store_size )
             {
+                *(p_extra++) = p_oggpacket->bytes >> 24;
+                *(p_extra++) = p_oggpacket->bytes >> 16;
                 *(p_extra++) = p_oggpacket->bytes >> 8;
                 *(p_extra++) = p_oggpacket->bytes & 0xFF;
             }
             memcpy( p_extra, p_oggpacket->packet, p_oggpacket->bytes );
-            p_stream->i_headers += p_oggpacket->bytes + (b_store_size ? 2 : 0) + (b_store_num_headers ? 1 : 0);
+            p_stream->i_headers += p_oggpacket->bytes + (b_store_size ? 4 : 0) + (b_store_num_headers ? 1 : 0);
 
             if( !p_stream->b_force_backup )
             {
diff --git a/modules/mux/ogg.c b/modules/mux/ogg.c
index b767ed6..45eb604 100644
--- a/modules/mux/ogg.c
+++ b/modules/mux/ogg.c
@@ -638,10 +638,12 @@ static block_t *OggCreateHeader( sout_mux_t *p_mux )
             p_extra = p_input->p_fmt->p_extra;
             i_extra = p_input->p_fmt->i_extra;
 
-            op.bytes = *(p_extra++) << 8;
+            op.bytes = *(p_extra++) << 24;
+            op.bytes |= *(p_extra++) << 16;
+            op.bytes |= *(p_extra++) << 8;
             op.bytes |= (*(p_extra++) & 0xFF);
             op.packet = p_extra;
-            i_extra -= (op.bytes + 2);
+            i_extra -= (op.bytes + 4);
             if( i_extra < 0 )
             {
                 msg_Err( p_mux, "header data corrupted");
@@ -730,19 +732,23 @@ static block_t *OggCreateHeader( sout_mux_t *p_mux )
             i_extra = p_input->p_fmt->i_extra;
 
             /* Skip 1 header */
-            op.bytes = *(p_extra++) << 8;
+            op.bytes = *(p_extra++) << 24;
+            op.bytes |= *(p_extra++) << 16;
+            op.bytes |= *(p_extra++) << 8;
             op.bytes |= (*(p_extra++) & 0xFF);
             op.packet = p_extra;
             p_extra += op.bytes;
-            i_extra -= (op.bytes + 2);
+            i_extra -= (op.bytes + 4);
 
             while( j-- )
             {
-                op.bytes = *(p_extra++) << 8;
+                op.bytes = *(p_extra++) << 24;
+                op.bytes |= *(p_extra++) << 16;
+                op.bytes |= *(p_extra++) << 8;
                 op.bytes |= (*(p_extra++) & 0xFF);
                 op.packet = p_extra;
                 p_extra += op.bytes;
-                i_extra -= (op.bytes + 2);
+                i_extra -= (op.bytes + 4);
                 if( i_extra < 0 )
                 {
                     msg_Err( p_mux, "header data corrupted");
-- 
1.6.0

